/****************************************************************************
**
** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the QtWebEngine module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPLv3 included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or later as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 2.0 requirements will be
** met: http://www.gnu.org/licenses/gpl-2.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QQUICKWEBENGINECERTIFICATEERROR_P_H
#define QQUICKWEBENGINECERTIFICATEERROR_P_H

#include <QObject>
#include "qquickwebengineview_p.h"

QT_BEGIN_NAMESPACE

class QQuickWebEngineCertificateErrorPrivate;
class CertificateErrorController;

class Q_WEBENGINE_EXPORT QQuickWebEngineCertificateError : public QObject {
    Q_OBJECT
    Q_PROPERTY(QUrl url READ url)
    Q_PROPERTY(Error error READ error)
    Q_PROPERTY(QString description READ description)
    Q_PROPERTY(bool overridable READ overridable)
    Q_ENUMS(Error)

public:

    // Keep this identical to CertificateErrorController::CertificateError, or add mapping layer.
    enum Error {
        SslPinnedKeyNotInCertificateChain = -150,
        CertificateCommonNameInvalid = -200,
        CertificateDateInvalid = -201,
        CertificateAuthorityInvalid = -202,
        CertificateContainsErrors = -203,
        CertificateNoRevocationMechanism = -204,
        CertificateUnableToCheckRevocation = -205,
        CertificateRevoked = -206,
        CertificateInvalid = -207,
        CertificateWeakSignatureAlgorithm = -208,
        CertificateNonUniqueName = -210,
        CertificateWeakKey = -211,
        CertificateNameConstraintViolation = -212,
    };

    QQuickWebEngineCertificateError(const QSharedPointer<CertificateErrorController> &controller, QObject *parent = 0);
    ~QQuickWebEngineCertificateError();

    Q_INVOKABLE void defer();
    Q_INVOKABLE void ignoreCertificateError();
    Q_INVOKABLE void rejectCertificate();
    QUrl url() const;
    Error error() const;
    QString description() const;
    bool overridable() const;
    bool deferred() const;

private:
    Q_DISABLE_COPY(QQuickWebEngineCertificateError)
    Q_DECLARE_PRIVATE(QQuickWebEngineCertificateError)
    QScopedPointer<QQuickWebEngineCertificateErrorPrivate> d_ptr;
};

QT_END_NAMESPACE

QML_DECLARE_TYPE(QQuickWebEngineCertificateError)

#endif // QQUICKWEBENGINECERTIFICATEERROR_P_H
